import de.fhpotsdam.unfolding.*;
import de.fhpotsdam.unfolding.geo.*;
import de.fhpotsdam.unfolding.utils.*;
import de.fhpotsdam.unfolding.providers.*;
import de.fhpotsdam.unfolding.data.*;
import de.fhpotsdam.unfolding.marker.*;
import java.util.List;

import net.java.games.input.*;
import org.gamecontrolplus.*;
import org.gamecontrolplus.gui.*;

import cc.arduino.*;
import processing.serial.*;
import java.util.Scanner;

Serial serialPort;  // Serial port object

UnfoldingMap map;
static final int map_size = 540;

ControlDevice cont;
ControlIO control;


PImage txtr;
float theta;
  


Location planeLocation;
float zoom;;
int xAxis;
int yAxis;
int zRotation;
int slider;
float Button2;
float Button3;
float Button4;
float Button5;
float Button6; boolean is6pressed;
float Hat;
int i_provider;
String telemetrie;
float[] a_temp = new float[700];
int[] a_pressure = new int[700];
float[] a_alt = new float[700];
float[] a_accelx = new float[350];
float[] a_accely = new float[350];
float[] a_accelz = new float[350];
float[] a_gyrox = new float[350];
float[] a_gyroy = new float[350];
float[] a_gyroz = new float[350];
List<AbstractMapProvider> providers;

ImageMarker imgMarker2;
SimpleLinesMarker path;

void set_loc(float lat, float lng) {
    imgMarker2.setLocation(new Location(lat,lng));
    planeLocation = new Location(lat,lng);
    path.addLocations(planeLocation);
}


void setup() {
  
  String portName = Arduino.list()[0]; 
  int baudRate = 9600; 
  serialPort = new Serial(this, portName, baudRate);
  serialPort.bufferUntil('\n');

  size(1280, 720, P3D);
  planeLocation = new Location(44.5f,26.1f);
  zoom = 11;
  i_provider = 0;
  path = new SimpleLinesMarker();
  path.setColor(color(44, 91, 167));
  path.setStrokeWeight(3);
  control = ControlIO.getInstance(this);
  cont = control.getMatchedDevice("FlightStickX_std");
  
  if (cont == null) {
    println("controler not found");
    System.exit(-1);
  }
  providers = new ArrayList<AbstractMapProvider>();
  providers.add(new OpenStreetMap.PositronMapProvider());
  providers.add(new Microsoft.RoadProvider());
  providers.add(new Microsoft.AerialProvider());
  providers.add(new Microsoft.HybridProvider());
  providers.add(new StamenMapProvider.WaterColor());
  providers.add(new StamenMapProvider.TonerLite());
  providers.add(new StamenMapProvider.TonerBackground());
  providers.add(new Google.GoogleMapProvider());
  providers.add(new Google.GoogleTerrainProvider());
                
  map = new UnfoldingMap(this, 10, 10, map_size+10, map_size+10);

  map.zoomAndPanTo((int)zoom, planeLocation);
  MapUtils.createDefaultEventDispatcher(this, map);
  is6pressed = false;
  
  imgMarker2 = new ImageMarker(planeLocation, loadImage("uip/plane.png"));
  map.addMarkers(imgMarker2);
  map.addMarkers(path);
  
  
  txtr = loadImage("plane.jpg");
}


public void getUserInput() {
  Button2 = cont.getButton("Button2").getValue();
  Button3 = cont.getButton("Button3").getValue();
  Button4 = cont.getButton("Button4").getValue();
  Button5 = cont.getButton("Button5").getValue();
  Button6 = cont.getButton("Button6").getValue();
  xAxis = int((cont.getSlider("XAxis").getValue()+1)/2*127)+128;
  yAxis= int((cont.getSlider("YAxis").getValue()+1)/2*127)+128;
  zRotation = int((cont.getSlider("ZRotation").getValue()+1)/2*127+128);
  slider= int((cont.getSlider("Slider").getValue()+1)/2*127)+128;
  if (Button6 == 0) is6pressed = false;
  Hat = cont.getHat("HatSwitch").getValue();
}


public void updateMap() {
  if (Button3==8) map.panTo(planeLocation);
  if (Hat==1) map.panTo(map.getLocation(map_size/2+10, map_size/2+10));
  if (Hat==2) map.panTo(map.getLocation(map_size/2+15, map_size/2+10));
  if (Hat==3) map.panTo(map.getLocation(map_size/2+20, map_size/2+10));
  if (Hat==4) map.panTo(map.getLocation(map_size/2+20, map_size/2+15));
  if (Hat==5) map.panTo(map.getLocation(map_size/2+20, map_size/2+20));
  if (Hat==6) map.panTo(map.getLocation(map_size/2+15, map_size/2+20));
  if (Hat==7) map.panTo(map.getLocation(map_size/2+10, map_size/2+20));
  if (Hat==8) map.panTo(map.getLocation(map_size/2+10, map_size/2+15));
  if(Button2 == 8) {
    
    if (zoom < 18.5f) zoom+=0.03;
    Location center_map = map.getLocation(map_size/2+15, map_size/2+15);
    map.zoomTo(zoom);
    map.panTo(center_map);
  }
  if(Button4 == 8) {
    
    if (zoom > 1) zoom-=0.03;
    Location center_map = map.getLocation(map_size/2+15, map_size/2+15);
    map.zoomTo(zoom);
    map.panTo(center_map);
  }
  if(!keyPressed) is6pressed =false;
  if(((keyPressed && key == '6') ||Button6 == 8) && is6pressed ==false){
    is6pressed = true;
    i_provider ++;
    i_provider%=providers.size();
    map.mapDisplay.setProvider(providers.get(i_provider));
  }
    
}

int mapper(float val, int upl, int downl, float maxf, float minf){
  float range = maxf - minf;
  float unit = (upl - downl)/range;
  return upl-(int)((val-minf)*unit);
}

void draw() {
  
  background(0);
  getUserInput();
  updateMap();
  map.draw();
  if(telemetrie!=null){
    Scanner scaner = new Scanner(telemetrie);
    scaner.useDelimiter(" |,");
    String str = scaner.next();
    str = scaner.next();str = scaner.next();str = scaner.next();
    float lat = Float.parseFloat(scaner.next());  
    str = scaner.next();
    float lng = Float.parseFloat(scaner.next());  
    
    if (lat+lng != 0)
      set_loc(lat, lng);
    
    str = scaner.next();
    String date = scaner.next();
    str = scaner.next();
    float temp = Float.parseFloat(scaner.next());  
    str = scaner.next();
    a_temp[695] = temp;
    int pressure = Integer.parseInt(scaner.next());  
    str = scaner.next();
    a_pressure[695] = pressure;
    float alt = Float.parseFloat(scaner.next());  
    str = scaner.next();
    a_alt[695] = alt;
    float accelx = Float.parseFloat(scaner.next());  
    str = scaner.next();
    a_accelx[345] = accelx;
    float accely = Float.parseFloat(scaner.next());  
    str = scaner.next();
    a_accely[345] = accely;
    float accelz = Float.parseFloat(scaner.next());  
    str = scaner.next();
    a_accelz[345] = accelz;
    float gyrox = Float.parseFloat(scaner.next());  
    str = scaner.next();
    a_gyrox[345] = gyrox;
    float gyroy = Float.parseFloat(scaner.next());  
    str = scaner.next();
    a_gyroy[345] = gyroy;
    float gyroz = Float.parseFloat(scaner.next());  
    a_gyroz[345] = gyroz;
    str = scaner.next();str = scaner.next();str = scaner.next();

    fill(0, 408, 612, 204);
    
    textSize(128);

    text(date, 20, 670);
    
    fill(0, 408, 100, 204);
    text(scaner.next()+" is " +scaner.next(), 500, 670);
  }
  
  strokeWeight(3);
  stroke(0, 0, 255, 255);
  for (int i=0; i<695; i++){
    a_temp[i] = a_temp[i+1];
    line(560 + i, mapper(a_temp[i],60,10,30,15), 561 + i, mapper(a_temp[i+1],60,10,30,15));
    a_pressure[i] = a_pressure[i+1];
    line(560 + i, mapper(a_pressure[i],190,100,100050,100150), 561 + i, mapper(a_pressure[i+1],190,100,100050,100150));
    
    a_alt[i] = a_alt[i+1];
    line(560 + i, mapper(a_alt[i],280,190,130,100), 561 + i, mapper(a_alt[i+1],280,190,130,100));
    
  }
  for (int i=0; i<345; i++){
    a_accelx[i] = a_accelx[i+1];
    line(560 + i, mapper(a_accelx[i],370,280,2,-2), 561 + i, mapper(a_accelx[i+1],370,280,2,-2));
    a_accely[i] = a_accelz[i+1];
    line(560 + i, mapper(a_accelz[i],460,370,2,-2), 561 + i, mapper(a_accelz[i+1],460,370,2,-2));
    a_accelz[i] = a_accelz[i+1];
    line(560 + i, mapper(a_accelz[i],550,460,2,-2), 561 + i, mapper(a_accelz[i+1],550,460,2,-2));
    a_gyrox[i] = a_gyrox[i+1];
    line(560 +350 + i, mapper(a_gyrox[i],370,280,40,-40), 561 +350 + i, mapper(a_gyrox[i+1],370,280,40,-40));
    a_gyroy[i] = a_gyroy[i+1];
    line(560 +350 + i, mapper(a_gyroy[i],460,370,40,-40), 561 +350 + i, mapper(a_gyroy[i+1],460,370,40,-40));
    a_gyroz[i] = a_gyroz[i+1];
    line(560 +350 + i, mapper(a_gyroz[i],550,460,40,-40), 561 +350 + i, mapper(a_gyroz[i+1],550,460,40,-40));
  }
  delay(50);
  strokeWeight(4);
  stroke(0, 0, 255, 255);
  line(560, 100, width-10, 100); // X-axis
  line(560, 190, width-10, 190); // X-axis
  line(560, 280, width-10, 280); // X-axis
  line(560, 370, width-10, 370); // X-axis
  line(560, 460, width-10, 460); // X-axis
  line(560, 550, width-10, 550); // X-axis
  line(560, 10, 560, 550); // Y-axis
  line(910, 280, 910, 550); // Y-axis
  
  fill(0, 612, 0, 204);
    
  textSize(20);

  text("Temperatura", 565, 960);
  text("Presiune", 565, 180);
  text("Altitudine", 565, 270);
  text("Acceleratie X", 565, 360);
  text("Acceleratie Y", 565, 450);
  text("Acceleratie Z", 565, 530);
  
  text("Gyro X", 915, 360);
  text("Gyro Y", 915, 450);
  text("Gyro Z", 915, 530);
}



void serialEvent(Serial p) {
  // Read data from the serial port
  String dataReceived = p.readStringUntil('\n');
  
  if (dataReceived != null) {
    dataReceived = trim(dataReceived);  // Remove any whitespace characters
    
    // Process the received data
    println("Received: " + dataReceived);
    telemetrie = dataReceived;
  }
}
